
import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.HttpHandler;
import com.sun.net.httpserver.HttpServer;

import java.io.*;
import java.net.InetSocketAddress;
import java.util.HashMap;
import java.util.Map;

/**
 * 有人云http推送demo
 * 提供url校验和接收http推送数据的示例
 *
 * @author linzhennan
 * @description
 * @date 2024-04-17 18:06
 **/
public class Demo {
    HttpServer httpServer;
    /**
     * http服务端口
     */
    private static final int PORT = 8888;

    public static void main(String[] args) throws IOException {
        new Demo();
    }

    Demo() throws IOException {
        httpServer = HttpServer.create(new InetSocketAddress(PORT), 0);
        //设置path,页面上的检验地址填http://{url}:{port}/usr
        httpServer.createContext("/usr", new HttpPushHandler());
        httpServer.start();
        System.out.println("server start,port:" + PORT);
    }

    /**
     * 消息处理
     */
    private class HttpPushHandler implements HttpHandler {
        @Override
        public void handle(HttpExchange exchange) throws IOException {
            if ("POST".equals(exchange.getRequestMethod())) {
                /*
                 post类型的为http推送过来的数据
                */
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(exchange.getRequestBody()));
                StringBuilder requestBodyContent = new StringBuilder();
                String line;
                //接收请求的body中内容
                while ((line = bufferedReader.readLine()) != null) {
                    requestBodyContent.append(line);
                }
                System.out.println("content:" + requestBodyContent);
                //回复200成功状态码
                exchange.sendResponseHeaders(200, 0);
                OutputStream outputStream = exchange.getResponseBody();
                outputStream.close();
            } else {
                /*
                 get类型的为url校验请求
                 */
                Map<String, String> params = splitQuery(exchange.getRequestURI().getQuery());
                String verify = params.get("verify");
                System.out.println("verify:" + verify);
                exchange.sendResponseHeaders(200, verify.length());
                OutputStream outputStream = exchange.getResponseBody();
                //verify内容写入回复的body
                outputStream.write(verify.getBytes());
                outputStream.close();
            }
        }
    }

    /**
     * 分割uri中内容
     *
     * @param query
     * @return
     */
    private Map<String, String> splitQuery(String query) {
        Map<String, String> queryPairs = new HashMap<>();
        String[] pairs = query.split("&");
        for (String pair : pairs) {
            int idx = pair.indexOf("=");
            if (idx > 0) {
                queryPairs.put(pair.substring(0, idx), pair.substring(idx + 1));
            }
        }
        return queryPairs;
    }
}
